<?php

class PgaProcessor
{
	/**
	 * @var PgaUtils
	 */
	private $utils;

	private $account_id;
	private $cert_path;
	private $currency;

	/**
	 * @var PgaOrder
	 */
	private $order;

	public function __construct( array $settings )
	{
		$this->account_id = $settings['account_id'];
		$this->cert_path  = $settings['cert_path'];
		$this->currency   = $settings['currency'];

		$this->utils = new PgaUtils();
	}

	/**
	 * Проверка возможности проведения платежа
	 *
	 * @param $order
	 */
	public function checkPaymentResponse( PgaOrder $order )
	{
		// Тело XML-ответа
		$sxe = new SimpleXMLElement( "<payment-avail-response></payment-avail-response>" );

		$result = $sxe->addChild( 'result' );

		if ( $order->post && ! $order->is_paid() )
		{
			$result->addChild( 'code', 1 );
			$result->addChild( 'desc', 'OK' );

			$sxe->addChild( 'merchant-trx', $order->getId() );

			$purchase = $sxe->addChild( 'purchase' );
			$purchase->addChild( 'shortDesc', 'Заказ №' . $order->getId() );
			$purchase->addChild( 'longDesc', 'Заказ №' . $order->getId() );

			$accountAmount = $purchase->addChild( 'account-amount' );
			$accountAmount->addChild( 'id', $this->account_id );
			$accountAmount->addChild( 'amount', number_format( $order->getTotal(), 2, '', '' ) );
			$accountAmount->addChild( 'currency', $this->currency );
			$accountAmount->addChild( 'exponent', '2' );

			// Делаем запись в заметках заказа
			$order->add_order_note( 'PGA: платежный шлюз успешно проверил возможность оплаты данного заказа' );

			WC_Gateway_PGA::log( 'Found order #' . $order->id );
		}
		else
		{
			$result->addChild( 'code', 2 );

			if ( ! $order->post )
			{
				$result->addChild( 'desc', 'Order not found' );

				WC_Gateway_PGA::log( 'Order not found! Order ID: ' . $order->id );
			}
			elseif ( $order->is_paid() )
			{
				$result->addChild( 'desc', 'Order is paid' );

				WC_Gateway_PGA::log( 'Order is already paid! Order ID: ' . $order->id );
			}
		}

		$response = trim( $sxe->asXML() );

		echo $response;
		exit();
	}

	/**
	 * Регистрация платежа
	 *
	 * @param PgaOrder $order
	 * @param $result_code
	 */
	public function registerPaymentResponse( PgaOrder $order, $result_code )
	{
		$this->order = $order;

		$sxe    = new SimpleXMLElement( "<register-payment-response></register-payment-response>" );
		$result = $sxe->addChild( 'result' );

		if ( ! $this->checkCertificate() )
		{
			$result->addChild( 'code', 2 );
			$result->addChild( 'desc', 'Unable to read local certificate' );

			$response = trim( $sxe->asXML() );

			echo $response;
			exit();
		}

		// $signature_check
		$signature_check = $this->checkSignature();

		if ( $signature_check )
		{
			$result->addChild( 'code', 1 );
			$result->addChild( 'desc', 'OK' );

			// 'удачный платеж'
			// Статус платежа '1' эквивалентен статусу заказа 'Оплачен'.
			if($result_code===1)
			{
				$order->payment_complete();

				$order_note = 'PGA: платеж проведен успешно';
			}
			// 'неудачный платеж'
			// Статус платежа '2' эквивалентен статусу заказа 'Отменен'.
			else
			{
				$order_note = 'PGA: платеж НЕ проведен!';
			}

			// Делаем запись в заметках заказа
			$this->order->add_order_note( $order_note );
		}
		else
		{
			$result->addChild( 'code', 2 );
			$result->addChild( 'desc', 'Wrong signature.' );
		}

		$response = trim( $sxe->asXML() );

		echo $response;
		exit();
	}

	/**
	 * Проверка подписи (ЭЦП) в запросе от эквайера
	 *
	 * @return int
	 */
	public function checkSignature()
	{
		$SERVER_PROTOCOL = ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] == 'on' ) ? "https://" : "http://";

		if ( $_SERVER['HTTP_HOST'] == 'dev.intervale.ru:443' )
		{
			$request = explode( '&signature=', 'https://dev.intervale.ru:33443' . $_SERVER['REQUEST_URI'] );
		} else
		{
			$request = explode( '&signature=', $SERVER_PROTOCOL . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'] );
		}

		$data      = $request[0];
		$signature = base64_decode( urldecode( $request[1] ) );

		// Находим сертификат эквайера
		$fp = fopen( $_SERVER['DOCUMENT_ROOT'] . $this->cert_path, "r" );

		if ( ! $fp || ! $cert = fread( $fp, 8192 ) )
		{
			WC_Gateway_PGA::log( 'WARNING: unable to locate or read certificate at ' . $_SERVER['DOCUMENT_ROOT'] . $this->cert_path . '!' );

			return false;
		}

		fclose( $fp );

		// Достаем из сертификата публичный ключ
		$public_key = openssl_pkey_get_public( $cert );

		// Проверяем подпись и освобождаем ключ
		$signature_check = openssl_verify( $data, $signature, $public_key, OPENSSL_ALGO_SHA1 );

		openssl_free_key( $public_key );

		if ( ! $signature_check )
		{
			$message = 'OPENSSL_VERIFY Error: the signature is WRONG!' . PHP_EOL
			           . 'HTTP_HOST: ' . $_SERVER['HTTP_HOST'] . PHP_EOL
			           . 'SERVER_PROTOCOL: ' . $SERVER_PROTOCOL . PHP_EOL
			           . 'REQUEST_URI: '. $_SERVER['REQUEST_URI'];

			WC_Gateway_PGA::log( $message );

			$order_note = 'PGA: OPENSSL_VERIFY Error! Подпись НЕ корректна!';
		}
		else
		{
			$order_note = 'PGA: подпись регистрации платежа проверена и является корректной';
		}

		// Делаем запись в заметках заказа
		$this->order->add_order_note( $order_note );

		return $signature_check;
	}

	/**
	 * Проверка существования и доступности сертификата эквайера.
	 * Убедимся, что пользователь залил серификат по фтп на сайт и корректно прописал путь до него
	 * в настройках платежного модуля.
	 */
	public function checkCertificate()
	{
		$cert = $_SERVER['DOCUMENT_ROOT'] . $this->cert_path;

		$res = ( is_file( $cert ) && file_exists( $cert ) && is_readable( $cert ) );

		if($res)
		{
			WC_Gateway_PGA::log( 'Certificate SUCCESSFULLY located and readable.' );

			// Делаем запись в заметках заказа
			$this->order->add_order_note( 'PGA: сертификат успешно проверен' );
		}
		else
		{
			WC_Gateway_PGA::log( 'WARNING: unable to locate or read certificate at ' . $cert . '!' );

			// Делаем запись в заметках заказа
			$this->order->add_order_note( 'PGA: сертификат не найден или не читаем!' );
		}

		return $res;
	}
}