<?php

/**
 * Plugin Name: WooCommerce PGA Payment Gateway
 * Description: Easily adds PGA payment gateway to the WooCommerce plugin so you can allow customers to checkout via credit card.
 * Version: 1.0.0
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

if ( ! class_exists( 'WC_Gateway_PGA_Bootstrap' ) )
{
	final class WC_Gateway_PGA_Bootstrap
	{
		/**
		 * WC_Gateway_PGA_Bootstrap version.
		 *
		 * @var string
		 */
		public $version = '1.0.2';

		public $db_version = '1.0';

		/**
		 * The single instance of the class.
		 *
		 * @var WooCommerce
		 * @since 2.1
		 */
		protected static $_instance = null;

		/**
		 * Main WC_Gateway_PGA_Bootstrap Instance.
		 *
		 * Ensures only one instance of WC_Gateway_PGA_Bootstrap is loaded or can be loaded.
		 *
		 * @static
		 * @return WC_Gateway_PGA_Bootstrap - Main instance.
		 */
		public static function instance() {
			if ( is_null( self::$_instance ) ) {
				self::$_instance = new self();
			}
			return self::$_instance;
		}

		public function __construct() {
			$this->define_constants();
			$this->includes();
			$this->init_hooks();

			add_action('plugins_loaded', array(&$this, 'plugins_loaded_handler')); //plugins loaded hook
		}

		function plugins_loaded_handler()
		{
			include_once( 'includes/class-wc-gateway-pga.php' );
			include_once( 'includes/class-pga-order.php' );
			include_once( 'includes/class-pga-utils.php' );
			include_once( 'includes/class-pga-processor.php' );

			add_filter( 'woocommerce_payment_gateways', array( &$this, 'init_pga_gateway' ) );
		}

		function init_pga_gateway($methods)
		{
			array_push($methods, 'WC_Gateway_PGA');
			return $methods;
		}

		/**
		 * Hook into actions and filters.
		 */
		private function init_hooks()
		{
			// http://wp.dev/wc-api/wc_pga_check/...
			// http://wp.dev/?wc-api=wc_pga_check&trx_id=...
			add_action( 'woocommerce_api_wc_pga_check', array( $this, 'check_request_handler' ) );

			// http://wp.dev/?wc-api=wc_pga_reg
			add_action( 'woocommerce_api_wc_pga_reg', array( $this, 'reg_request_handler' ) );
		}

		public function check_request_handler()
		{
			$order = new PgaOrder( (int) $_GET['o_order_id'] );

			$pgaProc = $this->get_pga_processor();

			if($pgaProc){
				$pgaProc->checkPaymentResponse($order);
			}
			else
				exit;
		}

		public function reg_request_handler()
		{
			$order = new PgaOrder( (int) $_GET['o_order_id'] );

			$pgaProc = $this->get_pga_processor();

			if($pgaProc){
				$pgaProc->registerPaymentResponse( $order, (int)$_GET['result_code'] );
			}
			else
				exit;
		}

		/**
		 * @return null|PgaProcessor
		 */
		private function get_pga_processor()
		{
			$wc = WC();

			foreach($wc->payment_gateways()->payment_gateways as $pg)
			{
				if($pg->id=='pga')
				{
					return new PgaProcessor($pg->settings);
				}
			}

			return null;
		}

		/**
		 * Include required core files used in admin and on the frontend.
		 */
		public function includes() {

			if ( $this->is_request( 'admin' ) ) {
				// $this->admin_includes();
			}

			if ( $this->is_request( 'frontend' ) ) {
				// $this->frontend_includes();
			}
		}

		/**
		 * Define Constants.
		 */
		private function define_constants() {
			$upload_dir = wp_upload_dir();

			$this->define( 'WC_PGA_PLUGIN_FILE', __FILE__ );
			$this->define( 'WC_PGA_PLUGIN_BASENAME', plugin_basename( __FILE__ ) );
			$this->define( 'WC_PGA_VERSION', $this->version );
			$this->define( 'WC_PGA_LOG_DIR', $upload_dir['basedir'] . '/wc-pga-logs/' );
			$this->define( 'WC_PGA_SESSION_CACHE_GROUP', 'wc_pga_session_id' );
		}

		/**
		 * Define constant if not already set.
		 *
		 * @param  string $name
		 * @param  string|bool $value
		 */
		private function define( $name, $value ) {
			if ( ! defined( $name ) ) {
				define( $name, $value );
			}
		}

		/**
		 * What type of request is this?
		 * string $type ajax, frontend or admin.
		 *
		 * @param $type
		 * @return bool
		 */
		private function is_request( $type ) {
			switch ( $type ) {
				case 'admin' :
					return is_admin();
				case 'ajax' :
					return defined( 'DOING_AJAX' );
				case 'cron' :
					return defined( 'DOING_CRON' );
				case 'frontend' :
				default:
					return ( ! is_admin() || defined( 'DOING_AJAX' ) ) && ! defined( 'DOING_CRON' );
			}
		}
	}
}

/**
 * Main instance of WC_Gateway_PGA_Bootstrap.
 *
 * Returns the main instance of WC_Gateway_PGA_Bootstrap to prevent the need to use globals.
 *
 * @since  2.1
 * @return WooCommerce
 */
function WCGatewayPGABootstrap() {
	return WC_Gateway_PGA_Bootstrap::instance();
}

// Global for backwards compatibility.
$GLOBALS['WC_Gateway_PGA_Bootstrap'] = WCGatewayPGABootstrap();